<!--[if IE]><script type="text/javascript" src="{{ STATIC_URL }}dashboard_app/js/excanvas.min.js"></script><![endif]-->
<script type="text/javascript" src="{{ STATIC_URL }}dashboard_app/js/jquery.rpc.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}dashboard_app/js/jquery.flot.min.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}dashboard_app/js/jquery.dashboard.js"></script>
{% include "dashboard_app/_extrahead.html" %}

{% for report in report_list %}
<h4>{{ report }}</h4>
{{ report.get_html|safe }}
{% empty %}
<p>Currently no reports are configured for
display on the front page</p>
{% endfor %}
