{% load i18n %}
<div id="lava-server-extension-navigation">
  <ul>
    <li><a class="lava-back" href="{% url lava.home %}"
      >{% trans "Back to LAVA" %}</a></li>
    <li><a href="{% url dashboard_app.views.image_status_list %}"
      >{% trans "Image Status" %}</a></li>
    <li><a href="{% url dashboard_app.views.testing_effort_list %}"
      >{% trans "Testing efforts" %}</a></li>
    <li><a href="{% url dashboard_app.views.bundle_stream_list %}"
      >{% trans "Bundle Streams" %}</a></li>
    <li><a href="{% url dashboard_app.views.test_list %}"
      >{% trans "Tests" %}</a></li>
    <li><a href="{% url dashboard_app.views.data_view_list %}"
      >{% trans "Data Views" %}</a></li>
    <li><a href="{% url dashboard_app.views.report_list %}"
      >{% trans "Reports" %}</a></li>
  </ul>
</div>
